/*
 * Copyright (C) 2020  Magnus Bergman <magnus@snisurset.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <stdlib.h>                     /* atoi() */

#include "common.h"
#include "convert.h"
#include "debug.h"
#include "nil-dither.h"
#include "nil-palette.h"
#include "nil-png-r.h"
#include "nil-png-w.h"


static int opt_channel_depth = 8;
static const char *opt_palette;
static nil_dither_type_t opt_dither = NIL_DITHER_JJN;


static int on_option(int opt, const char *arg){
    DEBUG_p("arg: '%s'",arg);
    switch(opt){
    case 'd':
        opt_channel_depth = atoi(arg);
        return 0;
    case 'p':
        opt_palette = arg;
        return 0;
    DITHER_PARSE
    default:
        return -1;
    }
}

int main(int argc, char **argv){
    static const convert_option_t options[] = {
        {"channel-depth",'d',"DEPTH","Use a channel depth of DEPTH (default is 8)"},
        {"palette",'p',"PALETTE","Use PALETTE (the default is black and white)"},
        DITHER_OPTIONS
        {NULL}
    };
    convert_t conv;
    nile_t ni;

    convert_init(&conv, argc, argv, "png", "png", NULL, options, on_option);

    nil_png_read(&ni,conv.in);
    //nil_png_save(&ni, "tmp-in.png");
    DEBUG_p("after save");
    if(opt_palette){
        DEBUG_p("loading palette '%s'",opt_palette);
        if(nil_palette_load(&ni, opt_palette)<0){
            DEBUG_p("load failed");
            return 1;
        }
        DEBUG_p("got palette with %d colors",ni.colors);
    }else{
        /* FIXME!! set predefined palette or calculate one. */
        nil_palette_bw(&ni);
    }
    dither(&ni, opt_dither, 0, 0, opt_channel_depth);
    //nil_png_save(&ni, "tmp-out.png");
    nil_png_write(&ni,conv.out);

    convert_done(&conv);
    return 0;
}
