/*
 * Copyright (C) 2020  Magnus Bergman <magnus@snisurset.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <stdlib.h>                     /* atoi() */

#include "convert.h"
#include "debug.h"
#include "nil-color.h"
#include "nil-dither.h"
#include "nil-palette.h"
#include "nil-png-r.h"
#include "nil-png-w.h"
#include "nil-quant.h"


static int opt_colors = 256;
static int opt_channel_depth = 8;


static int on_option(int opt, const char *arg){
    switch(opt){
    case 'c':
        opt_colors = atoi(arg);
        return 0;
    case 'd':
        opt_channel_depth = atoi(arg);
        return 0;
    default:
        return -1;
    }
}

int main(int argc, char **argv){
    static const convert_option_t options[] = {
        {"colors",'c',"N","Create a palette of N colors (default is 256)"},
        {"channel-depth",'d',"DEPTH","Use a channel depth of DEPTH (default is 8)"},
        {NULL}
    };
    convert_t conv;
    nile_t ni;

    convert_init(&conv, argc, argv, "png", "png", NULL, options, on_option);

    nil_png_read(&ni,conv.in);
    nil_color_drop_alpha(&ni);
    ni.format = NILE_FORMAT_r8g8b8;
    DEBUG_p("reducing to %d colors",opt_colors);
    nil_quant(&ni, opt_colors, opt_channel_depth);
    nil_palette_sort(&ni);
    nil_dither_none(&ni);
    nil_png_write(&ni,conv.out);

    convert_done(&conv);
    return 0;
}
