/*
 * Copyright (C) 2020  Magnus Bergman <magnus@snisurset.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <assert.h>

#include "common.h"


#ifndef ABS
# define ABS(a) ((a)<0?-(a):(a))
#endif


void dither(nile_t *ni,nil_dither_type_t type,int xoffs,int yoffs,int channel_depth){
    switch(type){
    case NIL_DITHER_NONE:
        nil_dither_none(ni);
        break;
    case NIL_DITHER_ORD4:
        nil_dither_ordered4x4(ni, xoffs, yoffs, channel_depth);
        break;
    case NIL_DITHER_ORD8:
        nil_dither_ordered8x8(ni, xoffs, yoffs, channel_depth);
        break;
    case NIL_DITHER_FS:
        nil_dither_floyd_steinberg(ni);
        break;
    case NIL_DITHER_JJN:
        nil_dither_jarvis_judice_ninke(ni);
        break;
    case NIL_DITHER_STUCKI:
        nil_dither_stucki(ni);
        break;
    }
}


double compare_planar(const nile_t *ni1, const nile_t *ni2){
    double diff = 0;
    const uint8_t *p1 = ni1->planar;
    int skip1 = nile_get_planar_skip(ni1);
    const uint8_t *p2 = ni2->planar;
    int skip2 = nile_get_planar_skip(ni2);
    int y = ni1->height;
    assert(ni1->width <= ni2->width && ni1->height <= ni2->height);
    while(y-->0){
        int x = ni1->width;
        while(x-->0){
            diff += nile_color_variance(p1, ni1->format, p2, ni2->format);
            p1 += ni1->bpc;
            p2 += ni2->bpc;
        }
        p1 += skip1;
        p2 += skip2;
    }
    return diff;
}
