/*
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This file is public domain.
 */
#ifndef heap_h
#define heap_h

typedef enum{
    HEAP_MIN,
    HEAP_MAX
}heap_type_t;

typedef int(*heap_compare_func_t)(const void*, const void*);
typedef void(*heap_notify_func_t)(void*);

typedef struct{
    /** Size of the allocated memory (in number of items). */
    size_t size;
    /** Number of items in the heap. */
    size_t count;
    void **data;
    heap_type_t type;
    heap_compare_func_t compare;
}heap_t;

heap_t *heap_init(heap_t*, heap_type_t, heap_compare_func_t);
void heap_done_full(heap_t*,heap_notify_func_t);
#define heap_done(h) heap_done_full(h, NULL)
void heap_add(heap_t*, void *);
#define heap_peak(h) ((h)->data[0])
void *heap_pop(heap_t*);

#endif /* heap_h */
