/*
 * Copyright (C) 2020  Magnus Bergman <magnus@snisurset.net>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */
#include "debug.h"
#include "nil-atarist.h"

#ifndef MIN
# define MIN(a,b) ((a)<(b)?(a):(b))
#endif


int nil_atarist_vmem_encode(const nile_t *ni, nil_atarist_vmem_t *vmem){
    const uint8_t *src = ni->indexed;
    int src_stride = nile_get_indexed_stride(ni);
    uint8_t *dst = vmem->screen;
    int dst_stride;
    int y;
    int i;
    DEBUG_p("image size: %dx%d",ni->width,ni->height);
    switch(vmem->resolution[1]){
    case 0:
        dst_stride = ni->width/2;
        for(y=0;y<ni->height;++y){
            int x;
            for(x=0;x<ni->width/16;++x){
                int plane;
                for(plane=0;plane<4;++plane){
                    uint8_t *b = dst+x*2*4+plane*2;
                    b[0] = 0;
                    b[1] = 0;
                    for(i=0;i<8;++i){
                        if(src[x*16+0+i] & (1<<plane))
                            b[0] |= 128>>i;
                        if(src[x*16+8+i] & (1<<plane))
                            b[1] |= 128>>i;
                    }
                }
            }
            src += src_stride;
            dst += dst_stride;
        }
        break;
    case 1:
        dst_stride = ni->width/4;
        for(y=0;y<ni->height;++y){
            int x;
            for(x=0;x<ni->width/16;++x){
                int plane;
                for(plane=0;plane<2;++plane){
                    uint8_t *b = dst+x*2*2+plane*2;
                    b[0] = 0;
                    b[1] = 0;
                    for(i=0;i<8;++i){
                        if(src[x*16+0+i] & (1<<plane))
                            b[0] |= 128>>i;
                        if(src[x*16+8+i] & (1<<plane))
                            b[1] |= 128>>i;
                    }
                }
            }
            src += src_stride;
            dst += dst_stride;
        }
        break;
    default:
        return -1;
    }

    for(i=0;i<MIN(16,ni->colors);++i){
        int r = ni->palette[i*ni->bpc+0]>>5;
        int g = ni->palette[i*ni->bpc+1]>>5;
        int b = ni->palette[i*ni->bpc+2]>>5;
        int rgb = r<<8|g<<4|b<<0;
        vmem->palette[i*2+0] = rgb >> 8;
        vmem->palette[i*2+1] = rgb & 255;
    }
    for(i=ni->colors;i<16;++i){
        vmem->palette[i*2+0] = 0;
        vmem->palette[i*2+1] = 0;
    }
    return 0;
}
