/*
 * Copyright (C) 2020  Magnus Bergman <magnus@snisurset.net>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */
#include <string.h>                     /* memcpy() */

#include "nil-blit.h"


static void _blit_block(uint8_t *dst, int dst_stride, uint8_t *src, int src_stride, int width, int height){
    while(height-->0){
        memcpy(dst,src,width);
        src += src_stride;
        dst += dst_stride;
    }
}


void nil_paste_indexed(nile_t *dst_ni, const nile_t *src_ni, int x, int y){
    int src_stride = nile_get_indexed_stride(src_ni);
    int dst_stride = nile_get_indexed_stride(dst_ni);
    uint8_t *dst = dst_ni->indexed + x + y*dst_stride;
    _blit_block(dst, dst_stride, src_ni->indexed, src_stride, src_ni->width, src_ni->height);
}


void nil_paste_planar(nile_t *dst_ni, const nile_t *src_ni, int x, int y){
    int src_stride = nile_get_planar_stride(src_ni);
    int dst_stride = nile_get_planar_stride(dst_ni);
    uint8_t *dst = dst_ni->planar + x*dst_ni->bpc + y*dst_stride;
    _blit_block(dst, dst_stride, src_ni->planar, src_stride, src_ni->width * src_ni->bpc, src_ni->height);
}


void nil_paste(nile_t *dst,const nile_t *src, int x, int y){
    if(src->indexed && dst->indexed)
        nil_paste_indexed(dst, src, x, y);
    if(src->planar && dst->planar)
        nil_paste_planar(dst, src, x, y);
}


void nil_paste_indexed_to_planar(nile_t *dst_ni, const nile_t *src_ni, int x, int y){
    int src_skip = nile_get_indexed_skip(src_ni);
    const uint8_t *src = src_ni->indexed;
    int dst_stride = nile_get_planar_stride(dst_ni);
    int dst_skip = dst_stride - src_ni->width*src_ni->bpc;
    uint8_t *dst = dst_ni->planar + x*dst_ni->bpc + y*dst_stride;
    y = src_ni->height;
    while(y-->0){
        x = src_ni->width;
        while(x-->0){
            memcpy(dst, src_ni->palette + *src * src_ni->bpc, src_ni->bpc);
            ++src;
            dst += dst_ni->bpc;
        }
        src += src_skip;
        dst += dst_skip;
    }
}
