/*
 * Copyright (C) 2020  Magnus Bergman <magnus@snisurset.net>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */
#include "nil-color.h"


typedef void(*operation_func_t)(uint8_t*,int);


static void _op_drop_alpha(uint8_t *p, int arg){
    p[arg] = 0;
}


static void _op_shift_down3(uint8_t *p, int arg){
    p[0] >>= arg;
    p[1] >>= arg;
    p[2] >>= arg;
}


static void _op_shift_up3(uint8_t *p, int arg){
    int r = p[0] << arg;
    int g = p[1] << arg;
    int b = p[2] << arg;
    int bits = 8-arg;
    while(bits < 8){
        r |= r>>bits;
        g |= g>>bits;
        b |= b>>bits;
        bits <<= 1;
    }
    p[0] = r;
    p[1] = g;
    p[2] = b;
}


static void _color_operation(nile_t *ni, operation_func_t op, int arg){
    uint8_t *p;
    if(ni->palette){
        int i;
        p = ni->palette;
        for(i=0;i<ni->colors;++i){
            op(p, arg);
            p += ni->bpc;
        }
    }
    if(ni->planar){
        int skip = nile_get_planar_skip(ni);
        int y;
        p = ni->planar;
        for(y=0;y<ni->height;++y){
            int x;
            for(x=0;x<ni->width;++x){
                op(p, arg);
                p += ni->bpc;
            }
            p += skip;
        }
    }
}


void nil_color_drop_alpha(nile_t *ni){
    int alpha_o = NILE_OFFSET_alpha(ni->format);
    if(NILE_CHANNEL_MISSING==alpha_o)
        return;
    _color_operation(ni, _op_drop_alpha, alpha_o);
}


void nil_color_shift_down(nile_t *ni, int steps){
    if(steps>0)
        _color_operation(ni, _op_shift_down3, steps);
}


void nil_color_shift_up(nile_t *ni, int steps){
    if(steps>0)
        _color_operation(ni, _op_shift_up3, steps);
}
