/*
 * Copyright (C) 2020  Magnus Bergman <magnus@snisurset.net>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */
#include <math.h>                       /* sqrt() */
#include <stdio.h>
#include <stdlib.h>                     /* qsort() */

#include "debug.h"
#include "nil-palette.h"

int nil_palette_load(nile_t *ni, const char *filename){
    FILE *f = fopen(filename,"r");
    int size;
    int res;
    if(!f)
        return -1;
    res = -1;
    if(fseek(f, 0, SEEK_END)<0)
        goto out;
    size = ftell(f);
    DEBUG_p("size = %d",size);
    if(fseek(f, 0, SEEK_SET)<0)
        goto out;
    DEBUG_p("bpc: %d, colors: %d",ni->bpc, ni->colors);
    nile_ensure_palette(ni, size/3, 3);
    if(fread(ni->palette, size, 1, f)<0)
        goto out;
    res = 0;
out:
    fclose(f);
    return res;
}


static double _color_rgb_lum(const uint8_t *col){
    return sqrt(0.241 * col[0] + 0.691 * col[1] + 0.068 * col[2]);
}


static int _color_compare(const void *data1, const void *data2){
    double lum1 = _color_rgb_lum(data1);
    double lum2 = _color_rgb_lum(data2);
    if(lum1 == lum2){
        return 0;
    }else if(lum1 > lum2){
        return 1;
    }else{
        return -1;
    }
}


void nil_palette_sort(nile_t *ni){
    qsort(ni->palette, ni->colors, ni->bpc, _color_compare);
}


void nil_palette_bw(nile_t *ni){
    static const uint8_t bw[] = {0,0,0,255,255,255};
    int bpc = ni->bpc;
    if(!bpc)
        bpc = 3;
    nile_ensure_palette(ni, 2, bpc);
    nile_copy_colors_rgb(ni->palette, ni->bpc, ni->format, bw, 3, NILE_FORMAT_r8g8b8, 2);
}
