/*
 * Copyright (C) 2020  Magnus Bergman <magnus@snisurset.net>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */
#include <png.h>
#include <string.h>                     /* memcpy() */

#include "nil-png-r.h"

int nil_png_read(nile_t *ni, FILE *f){
    png_structp png;
    png_infop info;
    unsigned char** row_pointers;
    png_uint_32 width;
    png_uint_32 height;
    int bit_depth;
    int color_type;
    int interlace_type;
    int compression_type;
    int filter_method;

    png = png_create_read_struct(PNG_LIBPNG_VER_STRING,NULL,NULL,NULL);
    if(!png)
        return -1;
    info = png_create_info_struct(png);
    if(!info){
        png_destroy_read_struct(&png,NULL,NULL);
        return -1;
    }

    png_init_io(png,f);
    png_read_png(png,info,PNG_TRANSFORM_STRIP_16|PNG_TRANSFORM_PACKING,NULL);
    row_pointers = png_get_rows(png,info);

    png_get_IHDR(png,info,&width,&height,
                 &bit_depth, &color_type,&interlace_type,
                 &compression_type,&filter_method);

    nile_init(ni);
    switch(color_type){
    case PNG_COLOR_TYPE_PALETTE:
        /* FIXME!! */
        return -1;
    case PNG_COLOR_TYPE_RGB:
        nile_ensure_planar(ni,width,height,3);
        ni->format = NILE_FORMAT_r8g8b8;
        {
            int stride = nile_get_planar_stride(ni);
            int bytes = width*ni->bpc;
            uint8_t *dst = ni->planar;
            int y;
            for(y=0;y<height;++y){
                memcpy(dst,row_pointers[y],bytes);
                dst += stride;
            }
        }
        break;
    case PNG_COLOR_TYPE_RGBA:
        nile_ensure_planar(ni,width,height,4);
        ni->format = NILE_FORMAT_r8g8b8a8;
        {
            int stride = nile_get_planar_stride(ni);
            int bytes = width * ni->bpc;
            uint8_t *dst = ni->planar;
            int y;
            for(y=0;y<height;++y){
                memcpy(dst,row_pointers[y],bytes);
                dst += stride;
            }
        }
        break;
    }
    png_destroy_read_struct(&png,&info,NULL);
    return 0;
}


int nil_png_load(nile_t *ni, const char *filename){
    FILE* f = fopen(filename,"r");
    int res;
    if(!f)
        return -1;
    res = nil_png_read(ni, f);
    fclose(f);
    return res;
}
