/*
 * Copyright (C) 2020  Magnus Bergman <magnus@snisurset.net>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */
#include "nil-resize.h"

int nil_resize(nile_t *dst_ni, nile_t *src, int width, int height){
    int src_stride = nile_get_planar_stride(src);
    //int dst_stride;
    int dst_skip;
    uint8_t *dst;
    int py;
    nile_init(dst_ni);
    nile_ensure_planar(dst_ni, width, height, src->bpc);
    dst_ni->format = src->format;
    //dst_stride = nile_get_planar_stride(dst_ni);
    dst_skip = nile_get_planar_skip(dst_ni);
    dst = dst_ni->planar;

    double xscale = (double)width / src->width;
    double yscale = (double)height / src->height;
    //double threshold = 0.5 / (xscale * yscale);
    double yend = 0.0;

    for(py=0; py < height; ++py){
        double ystart = yend;
        double xend = 0.0;
        int px;
        yend = (py + 1) / yscale;
        if (yend >= src->height)
            yend = src->height - 0.000001;
        for(px=0;px<width;++px){
            double xstart = xend;
            double channel[8];
            int i;
            for(i=0;i<src->bpc;++i)
                channel[i] = 0;
            int y;
            xend = (px + 1) / xscale;
            if(xend >= src->width)
                xend = src->width - 0.000001;
            for(y=(int)ystart;y<=(int)yend;++y){
                double yportion = 1.0;
                int x;
                if(y == (int)ystart)
                    yportion -= ystart - y;
                if(y == (int)yend)
                    yportion -= y+1 - yend;
                for(x=(int)xstart; x <= (int)xend; ++x){
                    double xportion = 1.0;
                    const uint8_t *p = src->planar + x*src->bpc + y*src_stride;
                    if (x == (int)xstart)
                        xportion -= xstart - x;
                    if (x == (int)xend)
                        xportion -= x+1 - xend;
                    for(i=0;i<src->bpc;++i)
                        channel[i] += p[i] * yportion * xportion;
                }
            }
            for(i=0;i<src->bpc;++i)
                dst[i] = channel[i]*xscale*yscale;
            dst += dst_ni->bpc;
        }
        dst += dst_skip;
    }
    return 0;
}
