/*
 * Copyright (C) 2020  Magnus Bergman <magnus@snisurset.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <stdio.h>
#include <string.h>                     /* memcpy() */

#include "common.h"
#include "convert.h"
#include "nile.h"
#include "nil-atarist.h"
#include "nil-palette.h"
#include "nil-png-r.h"
#include "nil-png-w.h"
#include "nil-quant.h"
#include "nil-resize.h"
#include "ni-dual.h"
#include "ni-duo.h"

#define WIDTH 416
#define HEIGHT 273


static nil_dither_type_t opt_dither = NIL_DITHER_JJN;
static const char *opt_palette;

static void _copy_palette(nile_t *dst, const nile_t *src){
#if 0
    nile_ensure_palette(dst, src->colors, 3);
    nile_copy_colors_rgb(dst->palette, 3, dst->format, src->palette, src->bpc, src->format, src->colors);
#else
    nile_ensure_palette(dst, src->colors, src->bpc);
    memcpy(dst->palette, src->palette, dst->colors * dst->bpc);
#endif
}


static void
_copy_planar(nile_t *dst_ni, const nile_t *src_ni){
    const uint8_t *src = src_ni->planar;
    int src_stride = nile_get_planar_stride(src_ni);
    uint8_t *dst;
    int dst_stride;
    int y;
    nile_ensure_planar(dst_ni, src_ni->width, src_ni->height, src_ni->bpc);
    dst = dst_ni->planar;
    dst_stride = nile_get_planar_stride(dst_ni);
    for(y=0;y<src_ni->height;++y){
        memcpy(dst, src, src_ni->width * src_ni->bpc);
        src += src_stride;
        dst += dst_stride;
    }
}


static int on_option(int opt, const char *arg){
    switch(opt){
    case 'p':
        opt_palette = arg;
        return 0;
    DITHER_PARSE
    default:
        return -1;
    }
}


int main(int argc, char **argv){
    static const convert_option_t options[] = {
        {"palette",'p',"PALETTE","use PALETTE instead of calculating one"},
        DITHER_OPTIONS
        {NULL}
    };
    convert_t conv;
    nile_t src;
    nile_t dst;
    nile_t dst1;
    nile_t dst2;
    nil_atarist_vmem_t vmem1;
    nil_atarist_vmem_t vmem2;

    convert_init(&conv, argc, argv, "png", "duo", NULL, options, on_option);

    nil_png_read(&src, conv.in);
    nil_resize(&dst,&src, WIDTH, HEIGHT);
    nile_done(&src);

    nile_init(&dst1);
    if(opt_palette){
        nil_palette_load(&dst1, opt_palette);
        dst1.format = NILE_FORMAT_r8g8b8;
    }else{
        //dst1.format = dst.format;
        dst1.format = NILE_FORMAT_r8g8b8;
        _copy_planar(&dst1, &dst);
        nil_quant(&dst1, 16, 4);
        nile_drop_planar(&dst1);
    }
    nile_ensure_indexed(&dst1, WIDTH, HEIGHT, 0);

    nile_init(&dst2);
    dst2.format = dst1.format;
    _copy_palette(&dst2, &dst1);
    nile_ensure_indexed(&dst2, WIDTH, HEIGHT, 0);

    ni_encode_dual(&dst1,&dst2,&dst,opt_dither);

#ifdef USE_DEBUG
    nil_png_save(&dst,"tmp-duo.png");
    nil_png_save(&dst1,"tmp-duo1.png");
    nil_png_save(&dst2,"tmp-duo2.png");
#endif
    nile_done(&dst);

    nil_atarist_vmem_encode(&dst1, &vmem1);
    nil_atarist_vmem_encode(&dst2, &vmem2);
    ni_duo_write(&vmem1, &vmem2, conv.out);
    nile_done(&dst1);
    nile_done(&dst2);

    return 0;
}
