/*
 * Copyright (C) 2020  Magnus Bergman <magnus@snisurset.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <stdio.h>
#include <string.h>                     /* memcpy() */

#include "common.h"
#include "convert.h"
#include "nile.h"
#include "nil-atarist.h"
#include "nil-dither.h"
#include "nil-palette.h"
#include "nil-png-r.h"
#include "nil-quant.h"
#include "nil-resize.h"
#include "ni-degas.h"


static nil_dither_type_t opt_dither = NIL_DITHER_JJN;
static const char *opt_palette;


static int on_option(int opt, const char *arg){
    switch(opt){
    case 'p':
        opt_palette = arg;
        return 0;
    DITHER_PARSE
    default:
        return -1;
    }
}


int main(int argc, char **argv){
    static const convert_option_t options[] = {
        {"palette",'p',"PALETTE","use PALETTE instead of calculating one"},
        DITHER_OPTIONS
        {NULL}
    };
    convert_t conv;
    nile_t src;
    nile_t dst;
    nil_atarist_vmem_t vmem;

    convert_init(&conv, argc, argv, "png", "pi2", "Degas medium resolution", options, on_option);

    nil_png_read(&src,conv.in);
    nil_resize(&dst,&src, 640, 200);
    if(opt_palette){
        nil_palette_load(&dst, opt_palette);
    }else{
        nil_quant(&dst, 4, 3);
        nil_palette_sort(&dst);
    }
    dither(&dst, opt_dither, 0, 0, 3);

    nil_atarist_set_resolution(&vmem, 1);
    nil_atarist_vmem_encode(&dst, &vmem);
    ni_degas_piX_write(&vmem, conv.out);

    convert_done(&conv);
    return 0;
}
